from genericpath import exists
from tkinter import N
from fpdf import FPDF
from py import process
from opreport.opsramp import env, metrics, objects, patching
import datetime
from dateutil.relativedelta import relativedelta
import sys
import json
import matplotlib.pyplot as plt
import  matplotlib.dates as mdates
from matplotlib.ticker import MaxNLocator
#import numpy as np
import pandas as pd
import array
from functools import reduce
import os
import re
import time

class PDF(FPDF):
    def __init__(self, client, start, end, bl_start, bl_end, logo, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = client
        self.start = start
        self.end = end
        self.bl_start = bl_start
        self.bl_end = bl_end
        self.logo = logo


    def header(self):
        logo_width = 25
        self.image(self.logo, 10, 8, logo_width)
        self.set_font('Helvetica', 'B', 12)
        self.cell(w=self.epw, txt=self.client['name'], align='R', ln=1)
        self.ln(5)
        x = self.get_x()
        y = self.get_y()
        self.set_line_width(.5)
        self.line(x, y, self.epw+10, y)


        """
        self.set_font('Helvetica', 'B', 11)
        self.cell(40, 5, "Baseline:")   
        self.set_font('Helvetica', '', 11)
        self.cell(40, 5, self.bl_start.strftime("%m/%d/%Y") + " - " + self.bl_end.strftime("%m/%d/%Y") , ln=1)
        """

        self.ln(5)
 
        self.set_fill_color(254,231,205)
        self.set_font('Helvetica', 'B', 1)



    # Page footer
    def footer(self):
        if self.page_no() > 1:
            self.set_y(-15)
            self.set_font('Helvetica', '', 7)
            self.cell(0, 10, f'Page {self.page_no()}/{{nb}}', align='C')

def report(openv, tenants, daterange, outdir, logo):
    (start, end, bl_start, bl_end, utc_offset) = daterange
    for (tnum,tenant) in enumerate(tenants):
        #if tnum > 1:
        #    continue
        indexes = []
        data = []
        print(f'Start processing tenant {tenant["uniqueId"]} - {tenant["name"]}')
        resources = objects.get_objects(openv, obtype='resources', tenant=tenant['uniqueId'], queryString='agentInstalled:true+resourceType:Windows,Linux')
        resources = list(filter(lambda r: ('status' in r and r['status'] == 'UP'), resources))
        if len(resources) == 0:
            print("Skipping tenant as there are no Windows/Linux agent-installed resources.")
            continue
        resources_by_id = {}
        for resource in resources:
            resources_by_id[resource['id']] = resource
        all_patches = pd.DataFrame(pd.json_normalize(patching.get_patches_on_resources(openv, tenant['uniqueId'])))
        count_needs_updates = 0
        
        missing_patches = pd.DataFrame()
        if len(all_patches) > 0:
            all_patches['resource.state'] = all_patches.apply(lambda row: resources_by_id[row['resource.id']]['state'] if row['resource.id'] in resources_by_id else 'MISSING', axis=1)
            all_patches['resource.status'] = all_patches.apply(lambda row: resources_by_id[row['resource.id']]['status'] if row['resource.id'] in resources_by_id else 'MISSING', axis=1)
            missing_patches = all_patches[(all_patches['resource.status'] == 'UP') & (all_patches['patchStatus'] == 'MISSING')]
            count_needs_updates = missing_patches.groupby('resource.id')['resource.id'].nunique().count()

        count_current = len(resources) - count_needs_updates
        pct_compliance = "N/A"
        if len(resources) > 0:
            pct_complete = f'{int(count_current/len(resources)*100)}%'


        count = [count_current, count_needs_updates]
        labels = ['Current', 'Needs Updates']
        plt.bar(labels, count, color=['#4590f7','#ea3323'])
        
        ax=plt.gca()
        ax.set_ylabel("Device Count")
        ax.yaxis.set_major_locator(MaxNLocator(integer=True))
        ax.grid(axis='y')
        fig = ax.get_figure()
        fig.suptitle('Device Update Status', fontsize=24)
        plt.xticks(fontsize=16)
        plt.ylabel('Device Count', fontsize=16)
        imagefile = f'{tenant["uniqueId"]}_patch_device_update_status.png'
        plt.savefig(imagefile)
        plt.clf()
        plt.close('all')

        #print(json.dumps(opmetrics, indent=2, sort_keys=False))
        #sys.exit(0)
        pdf = PDF(tenant, start, end, bl_start, bl_end, logo, 'P', 'mm', 'Letter')

        # get total page numbers
        pdf.alias_nb_pages()

        #
        # COVER PAGE
        #
        pdf.add_page()
        pdf.set_font('Helvetica', 'B', 25)
        titlewidth = 170
        pdf.set_y(pdf.eph/3)

        pdf.cell(w=pdf.epw, txt='MONTHLY PATCH', align='C', ln=1)
        pdf.cell(w=pdf.epw, txt='MANAGEMENT REPORT', align='C', ln=1)
        pdf.ln(60)
        
        pdf.set_font('Helvetica', 'B', 16)
        pdf.cell(w=pdf.epw, txt=tenant['name'], align='C', ln=1)   
        pdf.ln(10)

        pdf.set_font('Helvetica', 'B', 14)
        pdf.cell(w=pdf.epw,txt="For the period:", align='C', ln=1)
        pdf.ln(5)
        pdf.cell(w=pdf.epw, txt=start.strftime("%m/%d/%Y") + " - " + end.strftime("%m/%d/%Y"), align='C', ln=1) 

        #
        # PIE CHART AND PCT
        #
        pdf.add_page()
        pdf.set_auto_page_break(auto=True, margin=15)
        pdf.set_font('Helvetica', '', 14)
        pdf.image(imagefile, w=pdf.epw/2, x=0)
        os.remove(imagefile)
        pdf.set_xy(pdf.epw/2, 26)
        pdf.cell(w=pdf.epw/2, txt="Devices That Are Up To Date", align='C', ln=1)
        pdf.ln(20)
        pdf.set_x(pdf.epw/2)
        pdf.set_font('Helvetica', '', 70)
        pdf.cell(w=pdf.epw/2, txt=pct_complete, align='C', ln=1)

        #
        # DEVICES MISSING REQUIRED PATCHES
        #

        cell_height = 6
        pdf.cell(w=pdf.epw, txt="", ln=1)
        pdf.ln(20)
        pdf.set_font('Helvetica', 'B', 13)
        pdf.cell(w=pdf.epw, txt="Devices Missing Required Patches", align='L', ln=1)
        pdf.ln(5)
        missing_per_server = missing_patches.groupby(['resource.name', 'resource.resourceType'])['resource.name'].count() if len(missing_patches)>0 else {}
        pdf.set_font('Helvetica', 'B', 11)
        col_headers = ['Device Name', 'System Type', 'Total Missing', 'Category']
        for (idx,col_header) in enumerate(col_headers):
            ln = 1 if idx == len(col_headers)-1 else 3
            align = 'C' if col_header=='Totel Missing' else 'L'
            pdf.multi_cell(w=pdf.epw/len(col_headers), h=cell_height, txt=col_header, align=align, border='B', ln=ln)  
        pdf.set_font('Helvetica', '', 11)     
        for (grp, val) in missing_per_server.items():           
            pdf.multi_cell(w=pdf.epw/len(col_headers), h=cell_height, txt=grp[0], align='L', border='TB', ln=3)
            pdf.multi_cell(w=pdf.epw/len(col_headers), h=cell_height, txt=grp[1], align='L', border='TB', ln=3)
            pdf.multi_cell(w=pdf.epw/len(col_headers), h=cell_height, txt=str(val), align='C', border='TB', ln=3)
            pdf.multi_cell(w=pdf.epw/len(col_headers), h=cell_height, txt="Security Updates", align='L', border='TB', ln=1)

        
        #
        # PATCH INSTALLATION SUMMARY
        #

        pdf.add_page()
        pdf.set_fill_color(38,68,121)
        pdf.set_text_color(255,255,255)
        pdf.set_font('Helvetica', 'B', 16)
        pdf.cell(w=pdf.epw, txt="Patch Installation Summary", align='L', ln=1, fill=True)
        pdf.ln(5)

        installed_patches = []
        if len(all_patches) > 0:
            all_patches['installedTime'] = pd.to_datetime(all_patches['installedTime'], infer_datetime_format=True)
            installed_patches = all_patches[(all_patches['patchStatus'] == 'INSTALLED') & (pd.notnull(all_patches['installedTime']))]
            installed_patches = installed_patches[(pd.to_datetime(installed_patches['installedTime']).dt.date>=start.date()) & (pd.to_datetime(installed_patches['installedTime']).dt.date<end.date())]
            print(f'Number of installed patches = {len(installed_patches)}')
            if len(installed_patches) > 0:
                sumlines = {}
                sumlines["Total Windows Patches Installed"] = len(installed_patches[(installed_patches['resource.resourceType']=='Windows')].groupby(['patch.id'])['patch.id'])
                sumlines["Total Linux Patches Installed"] = len(installed_patches[(installed_patches['resource.resourceType']=='Linux')].groupby(['patch.id'])['patch.id'])
                sumlines["Total Patches Installed"] = sumlines["Total Windows Patches Installed"] + sumlines["Total Linux Patches Installed"]

                sumlines["Total Windows Systems Patched"] = len(installed_patches[(installed_patches['resource.resourceType']=='Windows')].groupby(['resource.id'])['resource.id'])
                sumlines["Total Linux Systems Patched"]  = len(installed_patches[(installed_patches['resource.resourceType']=='Linux')].groupby(['resource.id'])['resource.id'])
                sumlines["Total Systems Patched"]  = sumlines["Total Windows Systems Patched"]  + sumlines["Total Linux Systems Patched"]

                sumlines["Total Patches Failed"] = 0
                sumlines["Total Patches Declined"] = 0

                patchtypes=installed_patches.groupby(['patch.category','patch.id']).size().groupby('patch.category').count()
                total = sum(patchtypes)
                patchtypes.plot.pie(labels=None, autopct=lambda s: str(round(s * total / 100)))
                ax=plt.gca()
                ax.set_ylabel(None)
                fig = ax.get_figure()
                fig.subplots_adjust(bottom=0.2)
                fig.suptitle('Patch Installation by Category', fontsize=17, fontname="Helvetica", fontweight="bold")
                plt.legend(patchtypes.index, loc="lower center", frameon=False, bbox_to_anchor=(0.5,-.2))
                imagefile = f'{tenant["uniqueId"]}_patch_types.png'
                plt.savefig(imagefile)
                plt.clf()
                plt.close('all')

                pdf.set_text_color(0,0,0)
                pdf.set_font('Helvetica', '', 12)
                pdf.cell(w=pdf.epw/2, txt="Patch Installation Summary", align='C', ln=1)
                pdf.ln(5)
                topy = pdf.get_y()
                pdf.set_font('Helvetica', '', 10)
                cell_height = 6
                for (name, val) in sumlines.items():
                    w_name = pdf.get_string_width(name)
                    w_val = pdf.epw/2-w_name-12
                    border = 'B'
                    if name == 'Total Patches Declined':
                        border = 'T'
                    pdf.cell(w=w_name, h=cell_height, txt=name, align='L', ln=0, border=border)
                    pdf.cell(w=w_val, h=cell_height, txt=str(val), align='R', ln=1, border=border)
                bottomy = pdf.get_y()
                pdf.image(imagefile, x=pdf.epw/2, y=topy-10, w=pdf.epw/2+10)
                os.remove(imagefile)
                pdf.ln(50)
                pdf.set_font('Helvetica', '', 12)
                pdf.cell(w=pdf.epw, txt="No Declined Patches During Report Period", align='C', ln=1)
                pdf.ln(20)
                pdf.set_font('Helvetica', '', 12)
                pdf.cell(w=pdf.epw, txt="No Failed Patch Installations During Report Period", align='C', ln=1)


        if len(installed_patches) == 0:
                pdf.set_text_color(0,0,0)
                pdf.set_font('Helvetica', '', 12)
                pdf.cell(w=pdf.epw, txt="No Patches Installed During Report Period", align='C', ln=1)            

        print(f'Done processing tenant {tenant["uniqueId"]} - {tenant["name"]}')

        repdir = f'{outdir}{os.sep}{tenant["name"]}'
        if not os.path.exists(repdir):
            os.makedirs(repdir)
        pdf_file = "patch_management_report.pdf"
        pdf.output(f'{repdir}{os.sep}{pdf_file}')


if __name__ == "__main__":
    # execute only if run as a script
    report()


from genericpath import exists
from fpdf import FPDF
from py import process
from opreport.opsramp import env, metrics, objects
import datetime
from dateutil.relativedelta import relativedelta
import sys
import json
import matplotlib.pyplot as plt
import  matplotlib.dates as mdates
#import numpy as np
import pandas as pd
import array
from functools import reduce
import os
import re
import opreport.helpers as helpers

class PDF(FPDF):
    def __init__(self, client, start, end, bl_start, bl_end, logo, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resource = {'name': 'none'}
        self.client = client
        self.start = start
        self.end = end
        self.bl_start = bl_start
        self.bl_end = bl_end
        self.logo = logo


    def header(self):
        #self.add_font('DejaVu Sans', '', 'fonts/DejaVuSans.ttf', uni=True)
        #self.add_font('DejaVu Sans Bold', '', 'fonts/DejaVuSans-Bold.ttf', uni=True)
        if self.page_no() == 1:
            self.image(self.logo, 10, 8, 40)
            # font
            self.set_font('DejaVu Sans Bold', '', 16)
            # Padding
            self.ln(15)
            # Title
            self.cell(30, 5, 'Utilization Comparison Report', border=False, ln=1, align='L')
            # Line break
            self.ln(5)
            self.set_font('DejaVu Sans Bold', '', 11)
            self.cell(40, 5, "Customer:")
            self.set_font('DejaVu Sans', '', 11)
            self.cell(40, 5, self.client['name'], ln=1)   
            self.set_font('DejaVu Sans Bold', '', 11)
            self.cell(40, 5, "Period:")
            self.set_font('DejaVu Sans', '', 11)
            self.cell(40, 5, self.start.strftime("%m/%d/%Y") + " - " + self.end.strftime("%m/%d/%Y") , ln=1)
            self.set_font('DejaVu Sans Bold', '', 11)
            self.cell(40, 5, "Baseline:")   
            self.set_font('DejaVu Sans', '', 11)
            self.cell(40, 5, self.bl_start.strftime("%m/%d/%Y") + " - " + self.bl_end.strftime("%m/%d/%Y") , ln=1)
            self.ln(5)
 
        self.set_fill_color(254,231,205)
        self.set_font('DejaVu Sans Bold', '', 10)

        if self.resource['name'] == 'none':
            self.cell(0, 5, "No matching devices for this client", fill=True, align='C')
        else:
            for attr in ['name', 'model', 'ipAddress', 'osName', 'aliasName']:
                if attr not in self.resource:
                    self.resource[attr] = "Unknown"

        self.resource['osName'] = re.sub(r' [.\d]+ Build.*','', self.resource['osName'])
        cwidth = self.epw/4
        cheight = 5
        fsize = 9
        self.set_font('DejaVu Sans Bold', '', fsize)
        self.cell(cwidth*.65, cheight, "Device Name:", fill=True, border='LT')
        self.set_font('DejaVu Sans', '', fsize)
        self.cell(cwidth*1, cheight, txt=self.resource['name'], ln=0, fill=True, border='T')
        self.set_font('DejaVu Sans Bold', '', fsize)
        self.cell(cwidth*.6, cheight, "Model:", fill=True, border='T')
        self.set_font('DejaVu Sans', '', fsize)
        self.cell(cwidth*1.75, cheight, txt=self.resource['model'], ln=1, fill=True, border='RT')

        self.set_font('DejaVu Sans Bold', '', fsize)
        self.cell(cwidth*.65, cheight, "IP:", fill=True, border='L')
        self.set_font('DejaVu Sans', '', fsize)
        self.cell(cwidth*1, cheight, txt=self.resource['ipAddress'], ln=0, fill=True, border='')
        self.set_font('DejaVu Sans Bold', '', fsize)
        self.cell(cwidth*.6, cheight, "OS:", fill=True, border='')
        self.set_font('DejaVu Sans', '', fsize)
        self.cell(cwidth*1.75, cheight, txt=self.resource['osName'], ln=1, fill=True, border='R')

        self.set_font('DejaVu Sans Bold', '', fsize)
        self.cell(cwidth*.65, cheight, "Alias:", fill=True, border='LB')
        self.set_font('DejaVu Sans', '', fsize)
        self.cell(cwidth*1, cheight, txt=self.resource['aliasName'], ln=0, fill=True, border='B')
        self.set_font('DejaVu Sans Bold', '', fsize)
        self.cell(cwidth*.6, cheight, "Warranty Exp.:", fill=True, border='B')
        self.set_font('DejaVu Sans', '', fsize)
        self.cell(cwidth*1.75, cheight, txt='', ln=1, fill=True, border='RB')


    # Page footer
    def footer(self):
        #self.add_font('DejaVu Sans', '', 'fonts/DejaVuSans.ttf', uni=True)
        # Set position of the footer
        self.set_y(-15)
        # set font
        self.set_font('DejaVu Sans', '', 7)
        # Page number
        self.cell(0, 10, f'Page {self.page_no()}/{{nb}}', align='C')

def report(openv, tenants, daterange, outdir, logo):
    (start, end, bl_start, bl_end, utc_offset) = daterange

    images_per_line = 2
    for tenant in tenants:
        print("Tenant is " + tenant['uniqueId'] + " - " + tenant['name'])
        pdf = PDF(tenant, start, end, bl_start, bl_end, logo, 'P', 'mm', 'Letter')
        pdf.add_font('DejaVu Sans', '', 'fonts/DejaVuSans.ttf', uni=True)
        pdf.add_font('DejaVu Sans Bold', '', 'fonts/DejaVuSans-Bold.ttf', uni=True)

        # get total page numbers
        pdf.alias_nb_pages()

        pdf.set_auto_page_break(auto=True, margin=15)
        pdf.set_font('DejaVu Sans', '', 14)

        is_first_resource = True
        resources = objects.get_objects(openv, obtype='resources', tenant=tenant['uniqueId'], queryString='resourceType:Windows,Linux')
    
        rcount = 0
        rtotal = len(resources)
        print("Got " + str(rtotal) + " matching resources for tenant " + tenant['name'] + " before filtering for actives.")
        resources = list(filter(lambda r: ('status' in r and r['status'] == 'UP'), resources))
        rtotal = len(resources)
        print("After filtering device count is " + str(rtotal))
        if rtotal == 0:
            print("Skipping this client as there are 0 matching devies")
            continue
        for resource in resources:
            rcount +=1
            #if rcount > 5:
            #    break
            print("Processing resource " + str(rcount) + " of " + str(rtotal) + " - " + resource['name'])
            pdf.resource = resource
            pdf.add_page()

            images_this_line = 0
            current_y = pdf.get_y()

            aggregates = {}
            units = {}
            labels = {
                'system.cpu.usage.utilization': 'CPU Utilization',
                'system.disk.usage.utilization': 'Disk Utilization',
                'system.memory.usage.utilization': 'Memory Utilization',
                'system.disk.usage.freespace': 'Disk Free Space'
            }
            bl_avgs = {}
            metriclist = ['system.cpu.usage.utilization','system.disk.usage.utilization','system.memory.usage.utilization']
            #for metricname in ['system.cpu.usage.utilization','system.disk.usage.utilization','system.memory.usage.utilization', 'system.disk.usage.freespace'] :
            for (midx,metricname) in enumerate(metriclist) :
                aggregates[metricname] = {}
                print("  Metric: " + metricname)
                unit_label = ""
                dfs = []
                bl_dfs = []
                if images_this_line >= images_per_line:
                    images_this_line = 0
                else:
                    pdf.set_y(current_y)
                current_y = pdf.get_y()

                opmetrics = metrics.get_metricdatasets(openv,tenant['uniqueId'],resource['id'], metricname, int(start.timestamp()+utc_offset), int(end.timestamp()+utc_offset-1))
                bl_opmetrics = metrics.get_metricdatasets(openv,tenant['uniqueId'],resource['id'], metricname, int(bl_start.timestamp()+utc_offset), int(bl_end.timestamp()+utc_offset-1))
                if len(opmetrics) > 0  and 'message' not in opmetrics and 'data' in  opmetrics[0] and len(opmetrics[0]['data']) > 0 :
                    show_legend = True
                    if len(opmetrics) == 1:
                        show_legend = False

                    for instance in opmetrics:
                        if re.match(r'.*docker/overlay.*', instance['component']):
                            continue
                        units[metricname] = instance['unit']
                        unit_label = instance['unitLabel']
                        dfs.append(pd.DataFrame.from_dict(instance['data'], orient='index',columns=[instance['component']] ))

                    for bl_instance in bl_opmetrics:
                        if re.match(r'.*docker/overlay.*', bl_instance['component']):
                            continue
                        bl_dfs.append(pd.DataFrame.from_dict(bl_instance['data'], orient='index',columns=[bl_instance['component']] ))



                    df_merged = reduce(lambda  left,right: pd.concat([left,right],axis=1), dfs)
                    df_merged.index = pd.to_datetime(df_merged.index,unit='s')
                    df_merged = df_merged.sort_index()

                    bl_df_merged = reduce(lambda  left,right: pd.concat([left,right],axis=1), bl_dfs)
     
                    for col in df_merged.columns.to_list():
                        plt.plot_date(df_merged.index.to_pydatetime(), df_merged[col], fmt='-')
                    ax=plt.gca()
                    fig = ax.get_figure()
                    fig.subplots_adjust(bottom=0.2)
                    if show_legend:
                        fig.legend(df_merged.columns, loc="lower center", ncol=len(labels[metricname]), frameon=False)
                        #fig.legend(df_merged.columns, loc="lower center", ncol=len(labels[metricname]), bbox_to_anchor=(0.5, -1))
                    plt.title(labels[metricname])
                    plt.xticks(rotation = 45)
                    ax.xaxis.set_major_formatter(mdates.DateFormatter('%b %d'))
                    

                    aggregates[metricname]['min'] = df_merged.min(axis=0)
                    aggregates[metricname]['max'] = df_merged.max(axis=0)
                    aggregates[metricname]['mean'] = df_merged.mean(axis=0)
                    aggregates[metricname]['std'] = df_merged.std(axis=0)
                    bl_avgs[metricname] = bl_df_merged.mean(axis=0)

                    #ax.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))
                    ax.set_ylabel(unit_label)

                    imagefile = resource['id'] + "_" + metricname +'.png'
                    plt.savefig(imagefile)
                    plt.clf()
                    plt.close(fig)

                    #print(json.dumps(opmetrics, indent=2, sort_keys=False))
                    #sys.exit(0)

                    pdf.image(imagefile, w=pdf.epw/images_per_line, x=pdf.epw/images_per_line*images_this_line+10)
                    os.remove(imagefile)
                    images_this_line += 1
                else:
                    pdf.set_x(images_this_line*pdf.epw/images_per_line)
                    images_this_line += 1
                    if images_this_line == images_per_line :
                        ln =1
                    else:
                        ln = 0
                    pdf.set_font('DejaVu Sans Bold', '', 10)
                    pdf.set_x(pdf.get_x() + 15)
                    pdf.cell(txt="No data for " + metricname, w=pdf.epw/images_per_line-10, h=50, border=0, ln = ln, align='C')
                    if midx == len(metriclist) - 1:
                        pdf.ln(80)


            pdf.set_font("DejaVu Sans", size=7)
            line_height = pdf.font_size * 1.5
            col_width = pdf.epw / 5 # distribute content evenly

            pdf.ln(line_height)
            pdf.set_font('DejaVu Sans Bold', '', 7)
            pdf.set_fill_color(254,231,205)
            pdf.multi_cell(col_width*2, line_height*2, 'Details', border=1, ln=3, max_line_height=pdf.font_size, fill=True, align='C')
            pdf.multi_cell(col_width*.5, line_height*2, 'Lowest', border=1, ln=3, max_line_height=pdf.font_size, fill=True, align='C')
            pdf.multi_cell(col_width*.5, line_height*2, 'Highest', border=1, ln=3, max_line_height=pdf.font_size, fill=True, align='C')
            pdf.multi_cell(col_width*.5, line_height*2, 'Average', border=1, ln=3, max_line_height=pdf.font_size, fill=True, align='C')
            pdf.multi_cell(col_width*.5, line_height*2, 'Std. Dev.', border=1, ln=3, max_line_height=pdf.font_size, fill=True, align='C')
            pdf.multi_cell(col_width*.5, line_height*2, 'Baseline Average', border=1, ln=3, max_line_height=pdf.font_size, fill=True, align='C')
            pdf.multi_cell(col_width*.5, line_height*2, 'Differential', border=1, ln=3, max_line_height=pdf.font_size, fill=True, align='C')
            pdf.ln(line_height*2)
            pdf.set_font('DejaVu Sans', '', 7)
            for (metricname, aggs) in aggregates.items():
                if len(aggs) == 0:
                    continue
                num_components = len(aggs['max'])
                if num_components == 1:
                        pdf.multi_cell(col_width*2, line_height, labels[metricname], border=1, ln=3, max_line_height=pdf.font_size)
                        for (agg, vals) in aggs.items():
                                cellval = ""
                                if vals[0] is None:
                                    cellval = "-"
                                else:
                                    cellval = "{:.2f}".format(vals[0]) + units[metricname]
                                pdf.multi_cell(col_width*.5, line_height, cellval, border=1, ln=3, max_line_height=pdf.font_size, align='C')
                        baseavg = ""
                        differ = ""
                        trend = ""
                        differnum = None
                        if pd.isna(bl_avgs[metricname][0]) or bl_avgs[metricname][0] is None or aggs['mean'][0] is None:
                            baseavg = "-"
                            differ = "-"
                            trend = "-"
                        else:
                            baseavg = "{:.2f}".format(bl_avgs[metricname][0]) + units[metricname]
                            differ =  "{:.2f}".format(aggs['mean'][0] - bl_avgs[metricname][0]) + units[metricname]
                            differnum = aggs['mean'][0] - bl_avgs[metricname][0]
                        pdf.multi_cell(col_width*.5, line_height, baseavg, border=1, ln=3, max_line_height=pdf.font_size, align='C')
                        if differnum is not None and differnum > 0:
                            differ = "+" + differ + " \u25b2"
                            pdf.set_text_color(255,0,0)
                        elif differnum is not None and differnum < 0:
                            differ = differ + " \u25bc"
                            pdf.set_text_color(76,153,0)
                        pdf.multi_cell(col_width*.5, line_height, differ, border=1, ln=3, max_line_height=pdf.font_size, align='C')
                        pdf.set_text_color(0,0,0)
                        pdf.ln(line_height)
                elif num_components > 1:
                        for i in range(0,num_components):
                            comp = aggs['max'].keys()[i]
                            pdf.multi_cell(col_width*2, line_height, labels[metricname] + " (" + comp + ")", border=1, ln=3, max_line_height=pdf.font_size)
                            for (agg, vals) in aggs.items():
                                cellval = ""
                                if vals[comp] is None:
                                    cellval = "-"
                                else:
                                    cellval = "{:.2f}".format(vals[comp]) + units[metricname]
                                pdf.multi_cell(col_width*.5, line_height, cellval , border=1, ln=3, max_line_height=pdf.font_size, align='C')

                            baseavg = ""
                            differ = ""
                            trend = ""
                            differnum = None
                            if pd.isna(bl_avgs[metricname][comp]) or bl_avgs[metricname][comp] is None or aggs['mean'][comp] is None:
                                baseavg = "-"
                                differ = "-"
                                trend = "-"
                            else:
                                baseavg = "{:.2f}".format(bl_avgs[metricname][comp]) + units[metricname]
                                differnum = aggs['mean'][comp] - bl_avgs[metricname][comp]
                                differ =  "{:.2f}".format(differnum)
                            pdf.multi_cell(col_width*.5, line_height, baseavg, border=1, ln=3, max_line_height=pdf.font_size, align='C')
                            if differnum is not None and differnum > 0:
                                differ = "+" + differ + " \u25b2"
                                pdf.set_text_color(255,0,0)
                            elif differnum is not None and differnum < 0:
                                differ = differ + " \u25bc"
                                pdf.set_text_color(76,153,0)
                            pdf.multi_cell(col_width*.5, line_height, differ, border=1, ln=3, max_line_height=pdf.font_size, align='C')
                            pdf.set_text_color(0,0,0)
                            pdf.ln(line_height)

            print("Completed resource " + resource['name'])

        repdir = f'{outdir}{os.sep}{tenant["name"]}'
        if not os.path.exists(repdir):
            os.makedirs(repdir)
        pdf_file = "utilization_comparison_report.pdf"
        pdf.output(f'{repdir}{os.sep}{pdf_file}')

if __name__ == "__main__":
    # execute only if run as a script
    report()


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['opreport', 'opreport.helpers', 'opreport.opsramp', 'opreport.reports']

package_data = \
{'': ['*']}

install_requires = \
['DateTime>=4.4,<5.0',
 'PyYAML>=6.0,<7.0',
 'appdirs>=1.4.4,<2.0.0',
 'argparse>=1.4.0,<2.0.0',
 'fpdf2>=2.5.0,<3.0.0',
 'json5>=0.9.6,<0.10.0',
 'matplotlib>=3.5.1,<4.0.0',
 'numpy>=1.22.2,<2.0.0',
 'pandas>=1.4.1,<2.0.0',
 'requests>=2.27.1,<3.0.0']

entry_points = \
{'console_scripts': ['opreport = reportutil:main']}

setup_kwargs = {
    'name': 'opreport',
    'version': '0.1.1',
    'description': 'Some useful multi-tenant pdf reports for OpsRamp',
    'long_description': '# Multi-Client Reporting Utility for OpsRamp  \n\n## Purpose\n\nThis is a utility intended for use by OpsRamp customers needing to batch-generate some useful reports across all tenants in their partner instance.\n\n## First time execution\nThe first time it is run, it will look to see if the default environment config directory exists, and if it\'s not there it will create it.\n\nAfter that, you can either update the environments.yml file in the environment config directory with your partner-level OpsRamp API credentials, or you can specify API credentials on the command line.\n\n## Available Reports\nThe following reports are currently available.  Each report generates a separate report file for each tenant unless ptherwise specified:\n\n### Server Utilization Comparison\nThis report contains a page for each Windows or Linux server in the environment, with time-series graphs for the speficied period for CPU Utilization, File System Utilization, and Memory Utilization.  A table at the bottom of each page shows min, max, and average for the current period and increase/decrease trend from the baseline period.\n\n### Network Utilization\nThis is a fairly standard Top N report for Cisco device CPU Utilization, Memory Utilization, and Interface Utilization, Errors, etc.  It shows a time series only for the Top N items, followed by a detail report of all items sorted from highest to lowest.\n\n## Patch Management\nSummary of patches installed in the reporting period.\n\n\n## Remote Control Usage\nHistogram and detailed list of remote control console sessions that occurred in the reporting period.\n\n\n## Command line help:\n\n\n    usage: opreport [-h] [--env ENV] [--envfile ENVFILE] [--api_url API_URL] [--key KEY] [--secret SECRET] [--tenant TENANT] [--start START] [--end END] [--outdir OUTDIR]\n                    [--logo LOGO] [--secure SECURE] [--tenant_criteria TENANT_CRITERIA]\n                    {uc,nu,pm,rcu} ...\n\n        Multi-Client Reporting Utility for OpsRamp\n\n    positional arguments:\n    {uc,nu,pm,rcu}        Available commands\n        uc                  Generate Server Utilization Comparison Reports\n        nu                  Generate Network Utilization Reports\n        pm                  Generate Patch Management Reports\n        rcu                 Generate Remote Control Usage Reports\n\n    optional arguments:\n    -h, --help            show this help message and exit\n    --start START         Start date of reporting period in YYYY-MM-DD format (default=1st of prior month)\n    --end END             End date of reporting period in YYYY-MM-DD format (default=Last day of prior month)\n    --outdir OUTDIR       Directory where report output will be written (Default: ./output)\n    --logo LOGO           Logo image file for report headers (Default: /Users/michael.friedhoff/Library/Application Support/OpsRamp/logo.jpeg)\n    --secure SECURE       Whether or not to verify SSL cert (Default: True)\n    --tenant_criteria TENANT_CRITERIA\n                            Search criteria for which tenants to include as per https://develop.opsramp.com/tenancy-access-controls/tenants-orgid-clients-search (Default:\n                            "activeStatus:true", which will include all active tenants)\n\n    Use yaml environment file credentials:\n    --env ENV             Name of environment to use, referencing a named set of API credentials in environments.yml\n    --envfile ENVFILE     Location of environments YAML file to be used (Default: /Users/michael.friedhoff/Library/Application Support/OpsRamp/environments.yml)\n\n    Use command line credentials:\n    --api_url API_URL     Customer-specific API URL such as https://mycompany.api.opsramp.com\n    --key KEY             Authentication key\n    --secret SECRET       Authentication secret\n    --tenant TENANT       Authentication secret',
    'author': 'Michael Friedhoff',
    'author_email': 'michael.friedhoff@opsramp.com',
    'maintainer': 'Michael Friedhoff',
    'maintainer_email': 'michael.friedhoff@opsramp.com',
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
